<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Pages_Gallery extends Controller_Hijax implements HasMenu {

	public $template = 'pages/gallery';
	public $subtitle = 'Pics &amp; Videos';
	
	protected $id_separator = ':';
	
	private $table = 'gallery';
	
	public function action_index()
	{
		$this->template->menu_gallery = Menu::factory($this, 'gallery');
		$this->template->subtitle = $this->subtitle;
		
		$id = $this->request->param('id');

		$this->render_dynamic($this->load($id));
		
	}
	
	private function load($id)
	{
		if (isset($id))		
		{	
			$parts = explode($this->id_separator, $id);
			$identifier = array_shift($parts);
		
			$code = implode('-', $parts);
			
			if ($identifier === 'video')		
			{	
				return $this->video($code);	
			}		
			if ($identifier === 'photo')		
			{		
				return $this->photo($code);		
			}		
			return 'Unknown entry type requested';		
		}		
		else
		{		
			return $this->load_latest_item();			
		}
		
	}
	
	public function load_latest_item()
	{
		$isAjax = $this->request->is_ajax();

		$latest_node = Node::factory('gallery', NULL, array('name', 'date', 'code', 'type'))
			->find_all();

		if(isset($latest_node[0]))
		{
			$node = $latest_node[0];
				
			$type = $node->gallery->type;

			$code = $type === 'Y' ? 'video' : 'photo';
				
			$code .= $this->id_separator.$node->gallery->code;
			
			$uri = '/gallery/'.$code;

			if( ! $isAjax)
			{		
				Request::initial()->redirect($uri);			
			}			
			else
			{
				return $this->load($code);			
			}			
		}		
	}
	
	private function video($code)
	{
		$config = Kohana::$config->load('gallery');
	
		$video = "<div class=gcontent-holder><iframe title='YouTube video player' width='{$config['width']}px' height='{$config['height']}px' src='http://www.youtube.com/embed/{$code}' frameborder=0 allowfullscreen></iframe></div>";
		return array($this->subtitle, $video);
	}
	
	private function photo($code)
	{
		return array($this->subtitle, $this->gallery_content($code));
	}
	
	public function gallery_content($code)
	{
	
		$config = Kohana::$config->load('gallery');
		$feed_url = "http%3A%2F%2F{$config['domain']}.photobucket.com%2Falbums%2F{$config['directory']}%2F{$config['username']}%2F{$code}%2Ffeed.rss";
		$gallery_content = "
		<div class=gcontent-holder style='position: relative;'>
		<div style='position: absolute;'>
		<object>
		<param name='gallery' value='{$feed_url}' />
		<param name='allowFullscreen' value='false' />
	
		<embed width='{$config['width']}' height='{$config['height']}' src='http://static.pbsrc.com/flash/rss_slideshow.swf'
		flashvars='rssFeed={$feed_url}'
		type='application/x-shockwave-flash'
		wmode='transparent' />
	
		</object>
		</div>
		<!--[if IE]>
		<div id='Gallery-IE-warning'>
		<b>You appear to be using Internet Explorer.</b><br>
		Depending on your settings Internet Explorer may decide not to show you the gallery.<br>
		Try out <b>Google Chrome</b>, <b>Firefox</b>, or <b>Opera</b> for a better browsing experience.<br>
		</div>
		<![endif]-->
		</div>
		";
	
		return $gallery_content;
	}
	

	public function menu_data($identifier)
	{	
		return $this->get_table_data();
	}
	
	public function get_table_data()
	{
		$table = $this->table;
		return Node::factory($table, NULL)
		->order_by('date', 'DESC')
		->published()
		->find_all();
	}
	
	public function menu_item($entry, $identifier)
	{
		$concrete = $this->table;
		$model = $entry->$concrete;
		$display_date = date('j M Y', $entry->created);
		$date_time = date('Y-m-d', $entry->created);
		
		$type = $model->type;
		
		$identifier = $type === 'P' ? 'photo' : 'video';
		
		$href = '/gallery/'.$identifier;
		
		if($model->code !== '' )
		{
			$href .= $this->id_separator.$model->code;
		}
		
		return array(
				'id' 	=> null,
				'class' => "Gallery-$identifier-menu-item ".$identifier,
				'href'	=> $href,
				'text' 	=> "$entry->name <time datetime=$date_time>$display_date</time>",
		);
	}
	
	public function is_active($item, $current, $identifier)
	{
		return $current === $item['href'];
	}

}