<?php defined('SYSPATH') or die('No direct script access.');

class Converter {

	protected static $instance;
	
	protected $rate = 1;
	
	protected $default_currency, $price_currency, $symbols;
	
	public static function instance()
	{
		if(is_null(self::$instance))
		{
			self::$instance = new Converter();
			self::$instance->set_rate();
		}
		return self::$instance;
	}
	
	public function default_currency()
	{
		if(is_null($this->default_currency))
		{
			$this->default_currency = Kohana::$config->load('currency.default');
		}
		return $this->default_currency;
	}
	
	public function price_currency()
	{
		if(is_null($this->price_currency))
		{
			$this->price_currency = Kohana::$config->load('currency.price_currency');
		}
		return $this->price_currency;
	}
	
	
	public function get_currency_symbols()
	{
		if(is_null($this->symbols))
		{
			$this->symbols = Kohana::$config->load('currency.symbols');
		}
		return $this->symbols;
	}	
	
	public function currency_symbol()
	{
		return Arr::get($this->get_currency_symbols(), Kohana::$config->load('currency.default'));
	}
	
	function get_list($isVoucher = FALSE)
	{
		$diving_list = Kohana::$config->load('rates.diving_list');
		$diving_packages_list = Kohana::$config->load('rates.diving_packages_list');
		$padi_list = Kohana::$config->load('rates.padi_list');
		$bsac_list = Kohana::$config->load('rates.bsac_list');
	
		$list = array_merge($diving_list, $diving_packages_list, $padi_list,$bsac_list);
		
		if($isVoucher)
		{
			$result = array();
			
			foreach ($list as $row )
			{
				if((bool)$row[3])
				{
					$result[] = $row;
				}
			}
			return $result;
		}
		return $list;
	}
	
	function get_prices($currency = NULL)
	{
		$diving_prices = Kohana::$config->load('rates.diving_prices');
		$diving_packages_prices = Kohana::$config->load('rates.diving_packages_prices');
		$padi_prices = Kohana::$config->load('rates.padi_prices');
		$bsac_prices = Kohana::$config->load('rates.bsac_prices');
		$equipment_prices = Kohana::$config->load('rates.equipment_prices');

		$prices = array_merge($diving_prices, $diving_packages_prices, $padi_prices, $bsac_prices, $equipment_prices);
	
		return Converter::instance()->convert_prices($prices, $currency);
	}

	function get_items_with_prices($currency = NULL, $isVoucher = FALSE)
	{
		$diving_list = Kohana::$config->load('rates.diving_list');
		$diving_packages_list = Kohana::$config->load('rates.diving_packages_list');
		$padi_list = Kohana::$config->load('rates.padi_list');
		$bsac_list = Kohana::$config->load('rates.bsac_list');
		$equipment_list = Kohana::$config->load('rates.equipment_list');

		$result = array();
		
		$list = array(
            'Dives' => $diving_list,
            'Dive Packages' => $diving_packages_list,
            'PADI Courses' => $padi_list,
            'BSAC Courses' => $bsac_list,
            'Equipment' => $equipment_list,
        );
		$prices = $this->get_prices($currency);
		
		foreach ($list as $key => $value)
		{
			$result[] = array(
                'label' => $key,
            );
			foreach ($value as $row)
			{
				$id = $row[0];
				if(!is_null($id))
				{
					$name = $row[1];
					$price = $prices[$id];
					$desc = $row[2];
				
					$result[] = array
					(
							'id' => $id,
							'name' => $name,
							'price' => $price,
							'desc' => $desc,
					);
				}
			}
		}		
		return $result;
	}
	
	function calculate_total($items)
	{
		$prices  = $this->get_prices();
	
		$total = 0;
	
		foreach ($items as $key => $value)
		{
			$price = $prices[$key] * $value;
			$total += $price;
		}
		return $total;
	}
	
	function set_rate(){
	
		$default_currency = Kohana::$config->load('currency.default');
		$price_currency = Kohana::$config->load('currency.price_currency');
		$currency_controller = new Kohana_Controller_Currency(Request::initial(), Response::factory());
	
		if($default_currency != $price_currency){
			$curr = $currency_controller->convert($default_currency, $price_currency);
			if(isset($curr['rate']))
			{
				$this->rate = $curr['rate'];
			}
	
		}
	}
	
	function convert_prices($prices, $currency = NULL){
		foreach ($prices as $key => $value)
		{
			if($currency !== $this->price_currency())
			{
				$prices[$key] = $this->convert_value($value);
			}
		}
		return $prices;
	}
	
	function convert_value($value){
		$value = $value / $this->rate;
		return number_format($value, 2, '.', '');
	}
	
	public function get_user_array($post,$suffix)
	{
		$items = array();
		foreach ($post as $key => $value)
		{
			$pos = strrpos($key, $suffix);
			if($pos && $value)
			{
				$key = substr($key, 0, $pos);
				$items[$key] = $value;
			}
		}
		return $items;
	}
	
	public function get_table_data($user_items)
	{
		$items = array();
		foreach ($user_items as $key => $value)
		{
			$unit_price = $this->convert_value($this->get_price_from_prices($key));
			$quantity = $value;
			$name = $this->get_name_from_lists($key);
			$total = $unit_price * $quantity;
			$symbol = $this->currency_symbol();
			
			$row = array($name, $quantity, $symbol.$unit_price, $symbol.$total);
			$items[] = $row;
		}
		
		return $items;
	}
	
	public function get_name_from_lists($key)
	{
		$lists = array(
					Kohana::$config->load('rates.diving_list'),
					Kohana::$config->load('rates.diving_packages_list'),
					Kohana::$config->load('rates.padi_list'),
					Kohana::$config->load('rates.bsac_list'),
				);
		
		foreach ($lists as $list)
		{
			foreach ($list as $row)
			{
				if($row[0] == $key)
				{
					return $row[1];
				}
			}
		}
	}
	
	public function get_price_from_prices($key)
	{
		$prices = array(
					Kohana::$config->load('rates.diving_prices'),
					Kohana::$config->load('rates.diving_packages_prices'),
					Kohana::$config->load('rates.padi_prices'),
					Kohana::$config->load('rates.bsac_prices'),
				);		
		foreach ($prices as $item)
		{
			if(isset($item[$key]))
			{
				return $item[$key];
			}
		}
	}
}
