<?php defined('SYSPATH') or die('No direct script access.');
/**
* Assets loader.
*
* @package		MSS
* @category 	Assets
* @author		Paul Schwarz
* @copyright	&copy; 2011 Media &amp; Software Studio
* @version		1.0
*
*/

class Kohana_Controller_Assets extends Controller {

	public function action_index()
	{
		$module = $this->request->param('module');
		$path = $this->request->param('path');
		
		$this->config = Kohana::$config->load('assets');
		
		$file_path = $this->find_file($module, $path);
		
		$this->serve_file($file_path);
	}
	
	private function find_file($module, $path)
	{
		$assets_directory = $this->config->get('directory');
		
		if (strtolower($module) === 'application')
		{
			$assets_path = APPPATH.$assets_directory;
		}
		else 
		{
			$modules = Kohana::modules();
			if (array_key_exists($module, $modules))
			{
				$assets_path = Arr::get($modules, $module).$assets_directory;
			}
			else
			{
				throw new HTTP_Exception_404('Module "'.$module.'" could not be found');
			}
		}
		
		if (!is_dir($assets_path))
		{
			throw new HTTP_Exception_404('Assets directory for module "'.$module.'" does not exist');
		}
		
		$file_path = $assets_path.DIRECTORY_SEPARATOR.$path;
		
		if (!file_exists($file_path))
		{
			$ext = pathinfo($file_path, PATHINFO_EXTENSION);
			
			$image_exts = array ('gif', 'jpeg', 'png', 'swf', 'psd', 'bmp',
	            'tiff', 'tiff', 'jpc', 'jp2', 'jpf', 'jb2', 'swc',
	            'aiff', 'wbmp', 'xbm');
			
			if (in_array($ext, $image_exts))
			{
				$this->serve_404_image($path);
			}
			else
			{
				throw new HTTP_Exception_404('File not found "'.$path.'"');
			}
		}
		
		return $file_path;
	}
	
	private function serve_file($file_path)
	{
		$lifetime = $this->config->get('cache_lifetime');
		
		$headers = array();
		
		if ($lifetime > 0) 
		{
			$headers['Pragma'] 			= 'public';
			$headers['Cache-Control'] 	= "max-age=$lifetime";
			$headers['Expires'] 		= gmdate('D, d M Y H:i:s', time()+$lifetime) . ' GMT';
		}
		
		$ext = Arr::get(pathinfo($file_path), 'extension', 'html');
		
		$headers['Content-Type'] = File::mime_by_ext($ext);
		
		$this->response
			->headers($headers)
			->send_file($file_path, NULL, array('inline' => TRUE));
	}
	
	private function serve_404_image($path)
	{
		$image_404 = $this->config->get('image_404');
		if (file_exists($image_404))
		{
			Request::current()->response()->status(404)->send_file($image_404, NULL, array('inline' => TRUE));
		}
		else 
		{
			throw new HTTP_Exception_404('Image not found "'.$path.'"');
		}
	}
	
} // End Assets
