if (!Hijax.Abstract) throw "Requires: [Hijax.Abstract]";
/**
 * Hijax.HTML5 
 * 
 * HTML5's History API implementation.
 * 
 * @package 	Hijax
 * @depends		Hijax
 */
Hijax.HTML5 = new Class({
	
	Extends: Hijax.Abstract,
	
	initHistory: function()
	{
		Hijax.log.info('Initialising Hijax History in HTML5 mode.');
		
		// On first page load update the referrer to the current state (HTML5 only)
		this.updateReferrer();
	
		// Native javascript code that handles window's popstate.
		window.addEventListener('popstate', function(e){
			
			var href = location.pathname;
			
			if (this.initial)
			{
				// HTML5 is different in that its first load has the actual content
				// window.fireEvent('popstateinitial', e);
				/*
				 * HTML5 mode already has the right page so don't load via ajax.
				 * We still need to fire the historychange event though.
				 */
				this.fireHistoryChange(href);
			}
			else
			{
				/*
				 * If it is not the initial page load then just let the mechanism
				 * do its thing like normal.
				 */
				this.onStateChanged(href);
			}
			this.initial = false;
			
		}.bind(this));
	},
	
	handleAnchorClick: function(href)
	{
		// First push the pretty URL to the browser state
		window.history.pushState(null, null, href);

		// Then call onStateChanged
		this.onStateChanged(href);
	},
	
	updateReferrer: function()
	{
		this.referrer = window.location.pathname;
	},
	
	/**
	 * Redirect, if necessary, before initialising the Hijax system
	 * 
	 * If in Old mode, hijax wants to be on the base page from where it uses #'s.
	 *  
	 * If in HTML5 mode and someone has referred you to a link that looks like /#...
	 * then redirect to the new version.
	 */
	redirect: function()
	{
		var path = window.location.pathname;

		/*
		 * If someone visits the site with an Old browser and loads /#/contact and 
		 * sends the link to someone with an HTML5 browser, then redirect the user
		 * to /contact  
		 */
		// Get the hash and strip the leading # and / if there, so we have: contact
		var hash = window.location.hash.replace(/^#\/?/, '');
		
		if('/' === path && hash.length > 0)
		{
			// Make the URL look nice
			// If pushState wasn't called then the resulting URL would be /contact#
			history.pushState(null, null, hash);
			
			// Now do the actual redirect to /contact
			window.location.pathname = hash;

			/*
			 * A hard redirect just occurred, so return true to notify the caller 
			 * not to continue with any more javascript execution on this page.
			 * (This is because javascript doesn't have an exit statement like php)
			 */
			return true;
		} 
		else if (location.href.contains('#'))
		{
			// Make the URL look nice
			// If pushState wasn't called then the resulting URL would be /contact#
			history.pushState(null, null, '/');
		}
		
		/*
		 * No hard redirect occurred, so we're going to stay on this page and allow
		 * the history layer to take over navigation from here.
		 */
		return false;
	}
	
});