var JSLogger = {
	
	logLevels: {
		'DEBUG'  : 1,
		'INFO'   : 2,
		'WARN'   : 3,
		'ERROR'  : 4
	}
	
};

JSLogger.AbstractAppender = new Class({
	
	formatter: null,
	
	initialize: function(formatter){
		this.formatter = formatter;
	},
	
	log: function(clazz, level, msg){
		this.formatter.format(this, clazz, level, msg);
	},
	
	display: function(level, formatted){}
	
});


JSLogger.AbstractFormatter = new Class({
	
	defaultFormat: function(clazz, level){
		return this.date()+' '+level+' '+clazz;
	},
	
	date: function(){
		var d = new Date();
		return this.pad(d.getHours(), 2) + ':'
			+ this.pad(d.getMinutes(), 2) + ':'
			+ this.pad(d.getSeconds(), 2) + '.'
			+ this.pad(d.getMilliseconds(), 3);
	},

	pad: function(value, mask){
		var strval = ''+value;
		while(strval.length < mask){
			strval = '0'+strval;
		}
		return strval;
	}
	
});


JSLogger.Writer = new Class({
	
	appender: null,
	clazz: null,
	
	initialize: function(appender, clazz){
		this.appender = appender;
		this.clazz = clazz;
	},
	
	debug: function(msg){
		this.log('DEBUG', msg);
	},

	info: function(msg){
		this.log('INFO', msg);
	},

	warn: function(msg){
		this.log('WARN', msg);
	},

	error: function(msg){
		this.log('ERROR', msg);
	},

	log: function(level, msg){
		var requested = JSLogger.logLevels[level];
		var permitted = JSLogger.logLevels[JSLogger.levels[this.clazz]];
		
		if (requested >= permitted){
			this.appender.log(this.clazz, level, msg);
		}
	}
	
});


JSLogger.Logger = new Class({

	appender: null,

	initialize: function(appender){
		this.appender = appender;
	},

	from: function(clazz){
		this.clazz = clazz;
		return new JSLogger.Writer(this.appender, clazz);
	}
	
});
