<?php defined('SYSPATH') or die('No direct script access.');
/**
 * JSLogger
 *
 * @package    Kohana/JSLogger
 * @author     Paul Schwarz
 */
class Kohana_JSLogger {

	public static $instance;
	
	public static function instance()
	{
		if (Kohana_JSLogger::$instance === NULL)
		{
			Kohana_JSLogger::$instance = new Kohana_JSLogger;
		}
		return Kohana_JSLogger::$instance;
	}
	
	public function render ()
	{
		$this->config = Kohana::$config->load('jslogger');
		
		$appender_name = $this->get_appender_name();
		
		if (!empty($appender_name))
		{
			$formatter_name = $this->config->get('formatter');
			$javascript = $this->load_logger()
				.$this->load_appender($appender_name)
				.$this->load_formatter($formatter_name)
				.$this->bootstrap($appender_name, $formatter_name)
				.$this->load_levels();
		}
		else
		{
			$javascript = $this->disabled();
		}
		
		return $this->compress($javascript);
	}
	
	private function compress($buffer) {
        /* remove comments */
        // $buffer = preg_replace("/((?:\/\*(?:[^*]|(?:\*+[^*\/]))*\*+\/)|(?:\/\/.*))/", "", $buffer);
        /* remove tabs, spaces, newlines, etc. */
        $buffer = str_replace(array("\r\n","\r","\t","\n",'  ','    ','     '), '', $buffer);
        /* remove other spaces before/after ) */
        // $buffer = preg_replace(array('(( )+\))','(\)( )+)'), ')', $buffer);
        return $buffer;
    }
	
	private function disabled()
	{
//		return 'window.addEvent(\'domready\', function(){
//			window.$L={
//				debug: function(){},
//				info: function(){},
//				warn: function(){},
//				error: function(){},
//				from: function(){return $L;}
//			};
//		});';
		return '(function(){
				window.$L={
					debug: function(){},
					info: function(){},
					warn: function(){},
					error: function(){},
					from: function(){return $L;}
				};
			})();';
	}
	
	private function bootstrap($appender_name, $formatter_name)
	{
//		return 'window.addEvent(\'domready\', function(){
//			JSLogger.Formatter = JSLogger.'.$formatter_name.';
//			JSLogger.Appender = JSLogger.'.$appender_name.';
//			window.$L = new JSLogger.Logger(new JSLogger.Appender(new JSLogger.Formatter));
//		});';
	return '(function(){
				JSLogger.Formatter = JSLogger.'.$formatter_name.';
				JSLogger.Appender = JSLogger.'.$appender_name.';
				window.$L = new JSLogger.Logger(new JSLogger.Appender(new JSLogger.Formatter));
			})();';
	}
	
	private function load_logger()
	{
		return $this->file_contents('assets/js', 'JSLogger');
	}
	
	private function load_levels()
	{
		$levels_entries = $this->config->get('levels');
		$entries = array();
		foreach ($levels_entries as $key => $val)
		{
			$entries[] = "'$key':'$val'";
		}
		return 'JSLogger.levels = {'.PHP_EOL.implode(','.PHP_EOL, $entries).PHP_EOL.'};'.PHP_EOL;
	}
	
	private function get_appender_name()
	{
		$appenders = $this->config->get('appenders');
		
		$appender_key = NULL;
		switch (Kohana::$environment)
		{
			case Kohana::PRODUCTION:
				$appender_key = 'PRODUCTION';
				break;
			case Kohana::STAGING:
				$appender_key = 'STAGING';
				break;
			case Kohana::TESTING:
				$appender_key = 'TESTING';
				break;
			case Kohana::DEVELOPMENT:
				$appender_key = 'DEVELOPMENT';
				break;
		}
		return Arr::get($appenders, $appender_key);
	}
	
	private function load_appender($appender_name)
	{
		if (empty($appender_name))
		{
			throw new Kohana_Exception('JSLogger configuration error. No appender specified.');
		}
		return $this->file_contents('assets/js/appenders', $appender_name);
	}
	
	private function load_formatter($formatter_name)
	{
		if (empty($formatter_name))
		{
			throw new Kohana_Exception('JSLogger configuration error. No formatter specified.');
		}
		return $this->file_contents('assets/js/formatters', $formatter_name);
	}
	
	private function file_contents($dir, $file)
	{
		$filename = Kohana::find_file($dir, $file, 'js');
		if (file_exists($filename))
		{
			return file_get_contents($filename);
		}
		else 
		{
			throw new Kohana_Exception('JSLogger configuration error. Failed to load file ":file".', 
				array(':file' => $file));
		}
	}

} // End Kohana_JSLogger
