<?php defined('SYSPATH') or die('No direct script access.');
/**
* Minify util wrapper. 
* Utility function for generating group URIs in HTML files
* 
* <strong>Gotcha:</strong>
* <em>Versioning is controlled by a timestamp that is dependent on the latest
* filemtime of the files in the group. If you modify your group config by adding
* or removing sources from the group and the ...</em>
* 
*
* @package    	MSS
* @category		Minify
* @author     	Paul Schwarz
* @version 		1.0
* @copyright  	&copy; 2011 Media &amp; Software Studio
*/

class MinifyUtil {
	
	// Minify instance
	protected static $_instance;
	
	/**
	 * Singleton pattern
	 *
	 * @return Minify_Util
	 */
	static function instance()
	{
		if ( ! isset(MinifyUtil::$_instance))
		{
			// Load the configuration for this type
			$config = Kohana::$config->load('minify');
		
			// Create a new instance
			MinifyUtil::$_instance = new MinifyUtil($config);
		}
		
		return MinifyUtil::$_instance;
	}
	
	protected $_config;
	protected $_builds = array();
	
	/**
	 * Loads Session and configuration options.
	 *
	 * @return  void
	 */
	public function __construct($config = array())
	{
		// Save the config in the object
		$this->_config = $config;
	}
	
	
	/**
	 * Get a timestamped URI to a minified resource using the default Minify install
	 * 
	 * @param string $group_code a key from "groups" in config
	 * @return string URL of the minified group resource
	 */
	public function group($group_code)
	{
		// Configure
		Minify_Build::$ampersand = $this->_config->get('min_utility_ampersand');
		$min_baseurl = $this->_config->get('min_utility_baseurl');
		$forceAmpersand = $this->_config->get('min_utility_forceAmpersand');
		$path = $forceAmpersand	? "/g={$group_code}" : "/?g={$group_code}";
		
		/*
		 * http://code.google.com/p/minify/issues/detail?id=241
		 * 
		 * When Mr Clay addresses this issue then:
		 *   $this->_uri(...) 
		 * becomes
		 *   $build->uri($min_baseurl.$path, $forceAmpersand)
		 */
		return $this->_uri($group_code, $min_baseurl.$path, $forceAmpersand)
			.(Kohana::$environment !== Kohana::PRODUCTION? '&debug=1' : '');
	}
	
	/**
	 * This calls Minify_Build which handles filemtime logic.
	 * Uses Singleton Pattern to prevent redundant calls to Minify_Build.
	 *
	 * @param string $group_code a key from "groups" in config
	 * @return Minify_Build
	 * @private
	 */
	private function _Minify_getBuild($group_code)
	{
		if (! isset($this->_builds[$group_code])) 
		{
			$group = Arr::get($this->_config->get('groups'), $group_code);
			$this->_builds[$group_code] = new Minify_Build($group);
		}
		return $this->_builds[$group_code];
	}
	
	
	/**
	 * 
	 * Get a hashed URI
	 * 
	 * When Mr Clay addresses this issue then maybe this function becomes redundent
	 * http://code.google.com/p/minify/issues/detail?id=241
	 * 
     * @param string $uri
     * @param boolean $forceAmpersand (default = false) Force the use of ampersand 
     * to append the timestamp to the URI.
     * @return string
	 */
	private function _uri($group_code, $uri, $forceAmpersand = false) 
	{
		// Get the group
		$group = Arr::get($this->_config->get('groups'), $group_code);
		
		// Get the build
		$build = $this->_Minify_getBuild($group_code);
		
		// Separator
		$sep = ($forceAmpersand || strpos($uri, '?') !== false)	
			? Minify_Build::$ampersand	: '?';
		
		$group_modified = $build->lastModified;
		$group_serialised = serialize($group);
		$hash = md5($group_modified.$group_serialised);
		
		return "{$uri}{$sep}{$hash}";
	}
	
}

?>