<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 *
 * @author 		Paul Schwarz
 * @package    	MSS/Yahoo Currency
 * @category 	Currency
 * @version 	1.0
 */
class Kohana_Controller_Currency extends Controller {

	/**
	 * Any currency is available either from the cache or from Yahoo, eg.
	 *  /currency/USD/AUD
	 */
	public function action_index()
	{
		$currency_from = $this->request->param('from');
		$currency_to = $this->request->param('to');
		
		$result = $this->convert($currency_from, $currency_to);
		
		$this->response
			->headers('content-type', File::mime_by_ext('js'))
			->body(json_encode($result));
	}
	
	function convert($currency_from, $currency_to)
	{
		if ( ! (isset($currency_from) && ($currency_to)))
		{
			throw new Kohana_Exception('Usage: /currency/USD/AUD');
		}
		
		if ($currency_from == $currency_to)
		{
			$result = array(
				'rate'	=> 1,
				'age'	=> 0,
			);
		}
		else
		{
			// Try the cache first
			$result = $this->from_cache($currency_from, $currency_to);
				
			if (!$result)
			{
				// Not in cache so get from Yahoo
				$result = $this->from_yahoo($currency_from, $currency_to);
			}
		}
			
		return $this->format($result);
	}
	
	function format($result)
	{
		$seconds = Arr::get($result, 'age');
		
		$result['updated'] = Date::fuzzy_span(time() - $seconds);
		
		return $result;
	}
	
	protected function from_yahoo($currency_from, $currency_to)
	{
		// Obtain the quote from Yahoo
		$uri = __('http://quote.yahoo.com/d/quotes.csv?s=:currency_from:currency_to=X&f=l1&e=.csv', array(
			':currency_from'	=> $currency_from,
			':currency_to' 		=> $currency_to,
		));
		
		$fd = @ fopen ($uri, "r");
		$exchange_rate = @ fread($fd, 2000);
		@ fclose($fd);
		$exchange_rate = trim($exchange_rate);
		
		if( ! is_numeric($exchange_rate))
		{
			return null;
		}
		
		// Save the quote in the database
		$config = Kohana::$config->load('currency');
		$table = Arr::get($config, 'table', 'currency');
		
		DB::insert($table, array('from', 'to', 'rate'))
			->values(array($currency_from, $currency_to, $exchange_rate))
			->execute();
		
		// Return the quote information
		return array(
			'rate' 	=> $exchange_rate,
			'age'	=> 0,
		);
	}
	
	protected function from_cache($currency_from, $currency_to)
	{
		$config = Kohana::$config->load('currency');
		$max_age = Arr::get($config, 'max_age', 6);
		$table = Arr::get($config, 'table', 'currency');

		// First clear out any expired (older than max_age) cached results
		DB::delete($table)->where(
				DB::expr('DATE_ADD(`updated`, INTERVAL :max_age HOUR)', array(':max_age' => $max_age)), 
				'<', DB::expr('NOW()')
			)->execute();
		
		$result = DB::select('rate', array(DB::expr('TIME_TO_SEC(TIMEDIFF(NOW(), `updated`))'), 'age') )
			->from($table)
			->where('from', '=', $currency_from)
			->where('to', '=', $currency_to)
			->execute()
			->as_array();
		
		if (empty($result))
		{
			$result = DB::select('rate', array(DB::expr('TIME_TO_SEC(TIMEDIFF(NOW(), `updated`))'), 'age') )
				->from($table)
				->where('from', '=', $currency_to)
				->where('to', '=', $currency_from)
				->execute()
				->as_array();
			
			if ( ! empty($result))
			{
				$rate = $result[0];
				
				if (isset($rate['rate']) && is_numeric($rate['rate']) && $rate['rate'] != 0)
				{
					$result[0]['rate'] = 1 / $rate['rate'];
				}
			}
		}
		
		return Arr::get($result, 0);
	}
	
}
